==> seeds_widgets-2.0.8.zip: seeds_widgets/src/Plugin/Field/FieldFormatter/TimelineFormatter.php <==
<?php

namespace Drupal\seeds_widgets\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\seeds_widgets\SeedsWidgetsManager;
use Drupal\seeds_widgets\SeedsWidgetTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'timeline' formatter.
 *
 * @FieldFormatter(
 *   id = "seeds_widgets_timeline",
 *   label = @Translation("Seeds Widgets Timeline"),
 *   field_types = {
 *     "entity_reference_revisions"
 *   }
 * )
 */
class TimelineFormatter extends EntityReferenceFormatterBase implements ContainerFactoryPluginInterface, SeedsWidgetTypeInterface {

  /**
   * The Seeds Widgets Manager.
   *
   * @var \Drupal\seeds_widgets\SeedsWidgetsManager
   */
  protected $seedsWidgetsManager;

  /**
   * Constructs a TimelineFormatter object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\seeds_widgets\SeedsWidgetsManager $seeds_widgets_manager
   *   The Seeds Widgets Manager.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, SeedsWidgetsManager $seeds_widgets_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->seedsWidgetsManager = $seeds_widgets_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('seeds_widgets.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'alternate_sides' => TRUE,
      'animate' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    $settings = $this->getSettings();

    $form['alternate_sides'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Alternate Sides'),
      '#default_value' => $settings['alternate_sides'],
      '#description' => $this->t('Alternate content between left and right sides.'),
    ];

    $form['animate'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Animate'),
      '#default_value' => $settings['animate'],
      '#description' => $this->t('Add animation effects when timeline items come into view.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $settings = $this->getSettings();

    if ($settings['alternate_sides']) {
      $summary[] = $this->t('Alternating sides');
    }

    if ($settings['animate']) {
      $summary[] = $this->t('With animation');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    if ($items->isEmpty()) {
      return $elements;
    }

    $settings = $this->getSettings();
    $entities = $this->getEntitiesToView($items, $langcode);
    $processed_items = [];

    foreach ($entities as $delta => $entity) {
      if (!$entity) {
        continue;
      }

      $view_builder = \Drupal::entityTypeManager()->getViewBuilder($entity->getEntityTypeId());

      // Check if the paragraph has a custom view mode field and use that.
      $view_mode = $this->viewMode;
      if ($entity instanceof ContentEntityInterface &&
          $entity->hasField('field_seeds_widget_view_mode') &&
          !$entity->get('field_seeds_widget_view_mode')->isEmpty()) {
        $custom_view_mode = $entity->get('field_seeds_widget_view_mode')->target_id;
        if ($custom_view_mode) {
          $view_mode = str_replace("paragraph.", "", $custom_view_mode);
        }
      }

      $processed_items[$delta] = [
        'content' => $view_builder->view($entity, $view_mode, $entity->language()->getId()),
        '#paragraph' => $entity,
        'position' => $settings['alternate_sides'] ? ($delta % 2 ? 'right' : 'left') : 'left',
      ];
    }

    // Build the base render array with widget-specific theme.
    $elements[0] = [
      '#theme' => 'seeds_widgets_paragraph_formatter__timeline',
      '#items' => $processed_items,
      '#widget_type' => 'timeline',
      '#settings' => $settings,
      '#attached' => [
        'library' => ['seeds_widgets/timeline'],
      ],
    ];

    return $elements;
  }

}

==> seeds_widgets-2.5.1.zip: seeds_widgets/src/Plugin/Field/FieldFormatter/TimelineFormatter.php <==
<?php

namespace Drupal\seeds_widgets\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\seeds_widgets\SeedsWidgetsManager;
use Drupal\seeds_widgets\SeedsWidgetTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'timeline' formatter.
 *
 * @FieldFormatter(
 *   id = "seeds_widgets_timeline",
 *   label = @Translation("Seeds Widgets Timeline"),
 *   field_types = {
 *     "entity_reference_revisions"
 *   }
 * )
 */
class TimelineFormatter extends EntityReferenceFormatterBase implements ContainerFactoryPluginInterface, SeedsWidgetTypeInterface {

  /**
   * The Seeds Widgets Manager.
   *
   * @var \Drupal\seeds_widgets\SeedsWidgetsManager
   */
  protected $seedsWidgetsManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a TimelineFormatter object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\seeds_widgets\SeedsWidgetsManager $seeds_widgets_manager
   *   The Seeds Widgets Manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, SeedsWidgetsManager $seeds_widgets_manager, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->seedsWidgetsManager = $seeds_widgets_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('seeds_widgets.manager'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'alternate_sides' => TRUE,
      'animate' => TRUE,
      'orientation' => 'vertical',
      'marker_style' => 'default',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    $settings = $this->getSettings();

    $form['orientation'] = [
      '#type' => 'select',
      '#title' => $this->t('Orientation'),
      '#options' => [
        'vertical' => $this->t('Vertical'),
        'horizontal' => $this->t('Horizontal'),
      ],
      '#default_value' => $settings['orientation'],
      '#description' => $this->t('Select the layout orientation.'),
    ];

    $form['marker_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Marker Style'),
      '#options' => [
        'default' => $this->t('Default (Dot)'),
        'number' => $this->t('Number'),
        'check' => $this->t('Check Icon'),
      ],
      '#default_value' => $settings['marker_style'],
      '#description' => $this->t('Select the style of the timeline markers.'),
    ];

    $form['alternate_sides'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Alternate Sides'),
      '#default_value' => $settings['alternate_sides'],
      '#description' => $this->t('Alternate content between left and right sides (Vertical only).'),
      '#states' => [
        'visible' => [
          ':input[name*="orientation"]' => ['value' => 'vertical'],
        ],
      ],
    ];

    $form['animate'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Animate'),
      '#default_value' => $settings['animate'],
      '#description' => $this->t('Add animation effects when timeline items come into view.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $settings = $this->getSettings();

    $summary[] = $this->t('Orientation: @orientation', ['@orientation' => $settings['orientation']]);
    $summary[] = $this->t('Marker: @marker', ['@marker' => $settings['marker_style']]);

    if ($settings['orientation'] == 'vertical' && $settings['alternate_sides']) {
      $summary[] = $this->t('Alternating sides');
    }

    if ($settings['animate']) {
      $summary[] = $this->t('With animation');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    if ($items->isEmpty()) {
      return $elements;
    }

    $settings = $this->getSettings();
    $entities = $this->getEntitiesToView($items, $langcode);
    $processed_items = [];

    foreach ($entities as $delta => $entity) {
      if (!$entity) {
        continue;
      }

      $view_builder = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId());

      // Check if the paragraph has a custom view mode field and use that.
      $view_mode = $this->viewMode;
      if ($entity instanceof ContentEntityInterface &&
        $entity->hasField('field_seeds_widget_view_mode') &&
        !$entity->get('field_seeds_widget_view_mode')->isEmpty()) {
        $custom_view_mode = $entity->get('field_seeds_widget_view_mode')->target_id;
        if ($custom_view_mode) {
          $view_mode = str_replace("paragraph.", "", $custom_view_mode);
        }
      }

      $position = 'left';
      if ($settings['orientation'] == 'vertical' && $settings['alternate_sides']) {
        $position = ($delta % 2 ? 'right' : 'left');
      }

      $processed_items[$delta] = [
        'content' => $view_builder->view($entity, $view_mode, $entity->language()->getId()),
        '#paragraph' => $entity,
        'position' => $position,
      ];
    }

    // Build the base render array with widget-specific theme.
    $elements[0] = [
      '#theme' => 'seeds_widgets_paragraph_formatter__timeline',
      '#items' => $processed_items,
      '#widget_type' => 'timeline',
      '#settings' => $settings,
      '#attached' => [
        'library' => ['seeds_widgets/timeline'],
      ],
    ];

    return $elements;
  }

}
